<?php

namespace Drupal\survey\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\node\Entity\Node;

/**
 * Handles the survey results display for the Survey module.
 */
class SurveyResultController extends ControllerBase {

  /**
   * Displays the results of a single survey.
   *
   * @param \Drupal\node\Entity\Node $node
   *   The node entity for the survey.
   *
   * @return array
   *   A Drupal render array to display the survey results.
   */
  public function viewResults(Node $node) {
    if ($node->bundle() !== 'survey_content') {
      return [
        '#markup' => $this->t('Invalid survey content.'),
      ];
    }

    $build = [
      '#type' => 'container',
      '#attributes' => ['id' => 'survey-results'],
      '#attached' => [
        'library' => [
          'survey/chartjs',
          'survey/chart_drawer',
        ],
      ],
      '#cache' => ['max-age' => 0],
    ];

    $questions = $node->get('field_questions')->referencedEntities();
    $chart_data = [];
    foreach ($questions as $question) {
      $question_text = $question->get('field_question_text')->value;
      $response_data = $this->getResponseData($question->id());

      if (empty($response_data) || array_sum($response_data) == 0) {
        // If no responses, display a message instead of a chart.
        $build[] = [
          '#type' => 'markup',
          '#markup' => '<div><h3>' . $question_text . '</h3><p>0 responses found.</p></div>',
        ];
      }
      else {
        $labels = array_map(function ($label, $count) {
          return "$count responses for: $label";
        }, array_keys($response_data), array_values($response_data));

        $data = array_values($response_data);

        // Generate random colors for each answer.
        $backgroundColor = [];
        $borderColor = [];
        foreach ($response_data as $answer) {
          $randomColor = $this->generateRandomColor();
          $backgroundColor[] = $this->hexToRgba($randomColor, 0.2);
          $borderColor[] = $randomColor;
        }

        $canvas_id = 'chart-' . $question->id();
        $build[] = [
          '#type' => 'inline_template',
          '#template' => '<h3>{{ question_text }}</h3><div class="response-chart"><canvas id="{{ canvas_id }}" style="width: 834px; height: 400px;"></canvas></div>',
          '#context' => [
            'question_text' => $question_text,
            'canvas_id' => $canvas_id,
          ],
        ];

        $chart_data[$canvas_id] = [
          'type' => 'pie',
          'data' => [
            'labels' => $labels,
            'datasets' => [
              [
                'data' => $data,
                'backgroundColor' => $backgroundColor,
                'borderColor' => $borderColor,
                'borderWidth' => 1,
              ],
            ],
          ],
          'options' => [
            'responsive' => TRUE,
            'maintainAspectRatio' => FALSE,
            'legend' => [
                // Disable the default legend.
              'display' => FALSE,
            ],
            'plugins' => [
              'legend' => [
                'display' => TRUE,
                'position' => 'right',
                'labels' => [
                  'boxWidth' => 20,
                  'padding' => 20,
                ],
              ],
            ],
          ],
        ];
      }
    }

    $build['#attached']['drupalSettings']['survey']['charts'] = $chart_data;

    return $build;
  }

  /**
   * Retrieves response data for a specific survey question.
   *
   * This method fetches all answers associated with a specific question,
   * initializes a count of zero for each answer, and returns an array of
   * answer texts along with their corresponding response counts.
   *
   * @param int $question_id
   *   The ID of the question node from which to gather response data.
   *
   * @return array
   *   An associative array where each key is an answer ID, and each value is
   */
  private function getResponseData($question_id) {
    $question = Node::load($question_id);
    $answers = $question->get('field_answers')->referencedEntities();
    $all_answers = [];
    foreach ($answers as $answer) {
      $answer_id = $answer->id();
      $answer_text = $answer->get('field_answer_text')->value;
      $all_answers[$answer_id] = ['text' => $answer_text, 'count' => 0];
    }

    $query = \Drupal::database()->select('survey_responses', 'sr')
      ->fields('sr', ['answer_id'])
      ->condition('sr.question_id', $question_id)
      ->execute();

    foreach ($query as $record) {
      if (isset($all_answers[$record->answer_id])) {
        $all_answers[$record->answer_id]['count']++;
      }
    }

    $data = [];
    foreach ($all_answers as $answer) {
      $data[$answer['text']] = $answer['count'];
    }

    return $data;
  }

  /**
   * Generates a random hex color.
   *
   * @return string
   *   A hex color code.
   */
  private function generateRandomColor() {
    // Red.
    $r = dechex(mt_rand(0, 255));
    // Green.
    $g = dechex(mt_rand(0, 255));
    // Blue.
    $b = dechex(mt_rand(0, 255));
    return '#' . str_pad($r, 2, '0', STR_PAD_LEFT) . str_pad($g, 2, '0', STR_PAD_LEFT) . str_pad($b, 2, '0', STR_PAD_LEFT);
  }

  /**
   * Converts a hex color to an RGBA string.
   *
   * @param string $hex
   *   The hex color code.
   * @param float $alpha
   *   The alpha transparency value (from 0 to 1).
   *
   * @return string
   *   The RGBA color string.
   */
  private function hexToRgba($hex, $alpha) {
    [$r, $g, $b] = sscanf($hex, "#%02x%02x%02x");
    return "rgba($r, $g, $b, $alpha)";
  }

}
