<?php

namespace Drupal\survey\Entity\NodeType;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\node\Entity\NodeType;

/**
 * Create Question Content type.
 */
class QuestionContent {

  /**
   * Creates the 'Question Content' node type.
   *
   * This method initializes and saves a new node type called Question Content',
   * turning off the creation of new revisions, and setting up display settings.
   */
  public static function create() {
    $node_type = NodeType::create([
      'type' => 'question_content',
      'name' => 'Question Content',
      'status' => TRUE,
      'new_revision' => FALSE,
      'preview_mode' => 1,
      'display_submitted' => FALSE,
    ]);
    $node_type->set('options', ['promote' => FALSE]);
    $node_type->save();

    $fields = [
      'field_question_text' => [
        'type' => 'text_long',
        'settings' => [
          'allowed_formats' => ['basic_html'],
        ],
        'entity_type' => 'node',
        'bundle' => 'question_content',
        'label' => 'Question Text',
        'description' => 'The text of the question.',
        'required' => FALSE,
        'translatable' => TRUE,
        // Single value.
        'cardinality' => 1,
      ],
      'field_answers' => [
        'type' => 'entity_reference',
        'settings' => [
          'target_type' => 'node',
          'handler' => 'default:node',
          'handler_settings' => [
            'target_bundles' => ['answer_content' => 'answer_content'],
          ],
        ],
        'entity_type' => 'node',
        'bundle' => 'question_content',
        'label' => 'Answers',
        'description' => 'Add answers to this question.',
        'required' => FALSE,
        'translatable' => TRUE,
        // Unlimited.
        'cardinality' => -1,
      ],
    ];

    foreach ($fields as $field_name => $config) {
      $field_storage = FieldStorageConfig::create([
        'field_name' => $field_name,
        'type' => $config['type'],
        'entity_type' => $config['entity_type'],
        'settings' => $config['settings'],
        'cardinality' => $config['cardinality'],
      ]);
      $field_storage->save();

      $field_instance_settings = [];
      if ($config['type'] === 'entity_reference') {
        $field_instance_settings = [
          'handler' => 'default:node',
          'handler_settings' => [
            'target_bundles' => ['answer_content' => 'answer_content'],
          ],
        ];
      }

      $field_instance = FieldConfig::create([
        'field_storage' => $field_storage,
        'bundle' => $config['bundle'],
        'label' => $config['label'],
        'description' => $config['description'],
        'required' => $config['required'],
        'translatable' => $config['translatable'],
        'settings' => $field_instance_settings,
      ]);
      $field_instance->save();

    }
    $form_display = EntityFormDisplay::create([
      'targetEntityType' => 'node',
      'bundle' => 'question_content',
      'mode' => 'default',
      'status' => TRUE,
    ]);
    $form_display->setComponent('title', [
      'type' => 'string_textfield',
      'weight' => 0,
      'settings' => [
        'size' => 60,
        'placeholder' => '',
      ],
    ])->setComponent('field_question_text', [
      'type' => 'text_textarea',
      'weight' => 1,
      'settings' => [
        'rows' => 2,
        'placeholder' => '',
      ],
    ])->setComponent('field_answers', [
      'type' => 'inline_entity_form_complex',
      'weight' => 2,
      'settings' => [
        'form_mode' => 'default',
        'override_labels' => FALSE,
        'allow_new' => TRUE,
        'allow_existing' => FALSE,
        'match_operator' => 'CONTAINS',
        'allow_duplicate' => FALSE,
      ],
    ]);
    $form_display->save();

    $view_display = EntityViewDisplay::create([
      'targetEntityType' => 'node',
      'bundle' => 'question_content',
      'mode' => 'default',
      'status' => TRUE,
    ]);
    $view_display->setComponent('field_question_text', [
      'type' => 'text_default',
      'label' => 'hidden',
      'weight' => 1,
    ])->setComponent('field_answers', [
      'type' => 'entity_reference_entity_view',
      'label' => 'hidden',
      'settings' => [
        'view_mode' => 'default',
        'link' => TRUE,
      ],
      'weight' => 2,
    ]);
    $view_display->save();
  }

}
