<?php

namespace Drupal\survey\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;

/**
 * Provides a form for survey submissions.
 */
class SurveyForm extends FormBase {

  /**
   * The survey node.
   *
   * @var \Drupal\node\NodeInterface|null
   */
  protected ?NodeInterface $surveyNode;

  /**
   * Constructs a new SurveyForm.
   *
   * @param \Drupal\node\NodeInterface|null $surveyNode
   *   The survey node.
   */
  public function __construct(NodeInterface $surveyNode = NULL) {
    $this->surveyNode = $surveyNode;
  }

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'survey_form';
  }

  /**
   * Builds the survey form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param \Drupal\node\NodeInterface|null $node
   *   The survey node entity, if available.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state, NodeInterface $node = NULL) {
    $form['#attached']['library'][] = 'survey/custom_styles';
    $this->surveyNode = $node;
    // Get the current node from the route, if available.
    if ($this->surveyNode && $this->surveyNode->bundle() === 'survey_content') {
      $questions = $this->surveyNode->get('field_questions')->referencedEntities();
      // Add Bootstrap container class.
      $form['#attributes']['class'][] = 'container';
      foreach ($questions as $key => $question) {
        $answers = $question->get('field_answers')->referencedEntities();
        $options = [];
        foreach ($answers as $answer) {
          $options[$answer->id()] = $answer->get('field_answer_text')->value;
        }
        $question_number = $key + 1;
        $form['question_' . $key] = [
          '#type' => 'radios',
          '#title' => $this->t('@number. @title', [
            '@number' => $question_number,
            '@title' => $question->get('field_question_text')->value,
          ]),
          '#options' => $options,
          '#required' => TRUE,
          '#question_node_id' => $question->id(),
        // Add Bootstrap form-group class.
          '#prefix' => '<div class="form-group mb-4">',
          '#suffix' => '</div>',
        // Add Bootstrap form-check class.
          '#attributes' => ['class' => ['survey-form-check']],
          '#wrapper_attributes' => ['class' => ['survey-form-option']],
        ];
      }
      $form['survey_id'] = [
        '#type' => 'hidden',
        '#value' => $this->surveyNode->id(),
      ];
    }
    else {
      // Display a message if the node is not available.
      $form['message'] = [
        '#markup' => $this->t('No valid survey content found or the node is not of type "survey_content".'),
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    // Add Bootstrap button classes.
      '#attributes' => ['class' => ['btn', 'btn-primary']],
    // Add Bootstrap form-group class.
      '#prefix' => '<div class="form-group-button">',
      '#suffix' => '</div>',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $survey_id = $form_state->getValue('survey_id');
    $user_id = \Drupal::currentUser()->id();
    $current_time = REQUEST_TIME;

    foreach ($form_state->getValues() as $key => $value) {
      if (strpos($key, 'question_') === 0 && !empty($value)) {
        // Get the index from form element key.
        $question_index = substr($key, 9);
        // Fetch the Node ID from stored form element property.
        $question_node_id = $form['question_' . $question_index]['#question_node_id'];

        $answer_id = $value;
        if (!$answer_id) {
          // Skip if no answer is selected.
          continue;
        }

        // Insert data into the database.
        $insert_result = \Drupal::database()->insert('survey_responses')
          ->fields([
            'survey_id' => $survey_id,
            'question_id' => $question_node_id,
            'answer_id' => $answer_id,
            'user_id' => $user_id,
            'created' => $current_time,
          ])
          ->execute();

        // Log the insertion attempt.
        \Drupal::logger('survey_form')->notice('Inserted response with ID: @id', ['@id' => $insert_result]);
      }
    }
    $this->messenger()->addMessage($this->t('Thank you for your submission.'));
  }

}
